@@ -692,7 +692,8 @@ public class JvstTest4 extends JvstTestRoot {
         System.out.println("JIRA150: " + t1 + ", " + t2 + ", " + t3);
         assertTrue("performance test (the next try may succeed): " + t2 + " < 6 * " + t1,
                    t2 < t1 * 6);
-        assertTrue(t3 + " < 3 * " + t1, t3 < t1 * 3);
+        assertTrue("performance test (the next try may succeed): " + t3 + " < 3 * " + t1,
+                   t3 < t1 * 3);
     }
 
     public void testJIRA150b() throws Exception {
@@ -1054,7 +1055,7 @@ public class JvstTest4 extends JvstTestRoot {
         addDeadCode(newClass, "public boolean evaluate7(){ return !true; }");
 
         newClass.debugWriteFile();
-        Class<?> cClass = newClass.toClass();
+        Class<?> cClass = newClass.toClass(test4.DefineClassCapability.class);
         Object o = cClass.getConstructor().newInstance();
         java.lang.reflect.Method m = cClass.getMethod("evaluate");
         m.invoke(o);
@@ -1112,6 +1113,7 @@ public class JvstTest4 extends JvstTestRoot {
         attr.setAnnotation(a);
         m.getMethodInfo().addAttribute(attr);
         cc.writeFile();
+        anno.toClass(test4.DefineClassCapability.class);
         Class<?> rc = ((java.lang.annotation.Annotation)m.getAnnotations()[0]).annotationType();
         assertEquals(anno.getName(), rc.getName());
     }
