@@ -7,6 +7,7 @@ import junit.framework.TestCase;
 public class SetterTest extends TestCase {
 
     ClassPool pool;
+    Class<?> capability;
 
     public SetterTest(String name) {
          super(name);
@@ -15,6 +16,7 @@ public class SetterTest extends TestCase {
     protected void setUp() throws Exception {
         super.setUp();
         pool = ClassPool.getDefault();
+        capability = Class.forName("DefineClassCapability");
     }
 
     /**
@@ -28,7 +30,7 @@ public class SetterTest extends TestCase {
         CtField field = new CtField(CtClass.booleanType, "broken", clazz);
         clazz.addField(field, "true");
         clazz.addMethod(CtNewMethod.getter("isBroken", field));
-        Class _class = clazz.toClass();
+        Class _class = clazz.toClass(capability);
 
         Object object = _class.getConstructor().newInstance();
         check(_class, object, true);
@@ -46,7 +48,7 @@ public class SetterTest extends TestCase {
         clazz.addField(field, "true");
         clazz.addMethod(CtNewMethod.getter("isBroken", field));
         clazz.addMethod(CtNewMethod.setter("setBroken", field));
-        Class _class = clazz.toClass();
+        Class _class = clazz.toClass(capability);
 
         Object object = _class.getConstructor().newInstance();
 
@@ -66,7 +68,7 @@ public class SetterTest extends TestCase {
         field.setModifiers(Modifier.STATIC);
         clazz.addField(field, "true");
         clazz.addMethod(CtNewMethod.getter("isBroken", field));
-        Class _class = clazz.toClass();
+        Class _class = clazz.toClass(capability);
 
         Object object = _class.getConstructor().newInstance();
         check(_class, object, true);
@@ -85,7 +87,7 @@ public class SetterTest extends TestCase {
         clazz.addField(field, "true");
         clazz.addMethod(CtNewMethod.getter("isBroken", field));
         clazz.addMethod(CtNewMethod.setter("setBroken", field));
-        Class _class = clazz.toClass();
+        Class _class = clazz.toClass(capability);
 
         Object object = _class.getConstructor().newInstance();
 
